
from colibris import app
from colibris import persist

#
# def init(webapp, loop):
#     pass  # add your coroutines to the loop here
#
#
# def get_health():
#     # return your health status or raise app.HealthException()
#
#     if not persist.connectivity_check():
#         raise app.HealthException('database connectivity check failed')
#
#     return 'healthy'
#
