
DEBUG = True

LISTEN = '0.0.0.0'
PORT = 8888

# DATABASE = {
#     'backend': 'colibris.persist.PostgresqlDatabase',
#     'name': '__packagename__',
#     'host': '127.0.0.1',
#     'port': 5432,
#     'username': 'username',
#     'password': 'password'
# }

# AUTHENTICATION = {
#     'backend': 'colibris.authentication.jwt.JWTBackend',
#     'model': '__packagename__.models.User',
#     'identity_claim': 'sub',
#     'identity_field': 'username',
#     'secret_field': 'password'
# }

# AUTHORIZATION = {
#     'backend': 'colibris.authorization.role.RoleBackend',
#     'role_field': 'role'
# }
